@@ -20,7 +20,7 @@ use open ':std', $encoding;
 
 package App::GitGerrit;
 {
-  $App::GitGerrit::VERSION = '0.020';
+  $App::GitGerrit::VERSION = '0.021';
 }
 # ABSTRACT: A container for functions for the git-gerrit program
 
@@ -37,7 +37,7 @@ our @EXPORT_OK = qw/run/;
 # The $Command variable holds the name of the git-gerrit sub-command
 # that's been invoked. It's defined in the 'run' routine below.
 
-my $Command;
+our $Command;
 
 # The %Options hash is used to hold the command line options passed to
 # all git-gerrit subcommands. The --debug option is common to all of
@@ -573,18 +573,32 @@ sub auto_reviewers {
 
 # This routine is used by all sub-commands that accept zero or more
 # change ids. If @ARGV is empty it pushes into it the id of the change
-# associated with the current change-branch, if any.
+# associated with the current change-branch, if any. It returns a
+# boolean telling if the push has been made.
 
 sub grok_unspecified_change {
-    unless (@ARGV) {
+    if (@ARGV) {
+        return 0;
+    } else {
         my $id = current_change_id()
             or syntax_error "$Command: You have to be in a change-branch or specify at least one CHANGE.";
         $id =~ /^\d+$/
             or error "$Command: The change-branch you're in haven't been pushed yet.";
         @ARGV = ($id);
+        return 1;
     }
 }
 
+# This routine is used by the sub-commands that, when applied
+# successfully to the current change-branch, want to checkout its
+# upstream and remove the change-branch.
+
+sub checkout_upstream_and_delete_branch {
+    my $branch     = current_branch;
+    my ($upstream) = change_branch_info($branch);
+    cmd "git checkout $upstream" and cmd "git branch -D $branch";
+}
+
 ############################################################
 # MAIN
 
@@ -803,7 +817,7 @@ $Commands{my} = sub {
 $Commands{show} = sub {
     get_options();
 
-    grok_unspecified_change();
+    grok_unspecified_change;
 
     foreach my $id (@ARGV) {
         my $change = gerrit_or_die(GET => "/changes/$id/detail");
@@ -852,13 +866,14 @@ EOF
     return;
 };
 
-$Commands{checkout} = $Commands{co} = sub {
+$Commands{fetch} = sub {
     get_options();
 
-    grok_unspecified_change();
+    grok_unspecified_change;
 
     my $branch;
     my $project = config('project');
+    my @change_branches;
     foreach my $id (@ARGV) {
         my $change = get_change($id);
 
@@ -873,8 +888,21 @@ $Commands{checkout} = $Commands{co} = sub {
 
         cmd "git fetch $url $ref:$branch"
             or error "$Command: Can't fetch $url";
+
+        push @change_branches, $branch;
     }
-    cmd "git checkout $branch";
+
+    return @change_branches;
+};
+
+$Commands{checkout} = $Commands{co} = sub {
+    my $last_change_branch = do {
+        local $Command = 'fetch';
+        my @change_branches = $Commands{fetch}->();
+        $change_branches[-1];
+    };
+
+    cmd "git checkout $last_change_branch";
 
     return;
 };
@@ -903,34 +931,23 @@ $Commands{upstream} = $Commands{up} = sub {
 };
 
 $Commands{'cherry-pick'} = $Commands{cp} = sub {
-    # The 'gerrit cherry-pick' sub-command passes all of its options,
-    # but --debug, to 'git cherry-pick'.
-    Getopt::Long::Configure('pass_through');
-    get_options();
+    get_options(
+        'edit',
+        'no-commit',
+    );
 
-    # Since we're passing through options, they're left at the start
-    # of @ARGV. So, we pop the change-id instead of shifting it.
-    my $id = pop @ARGV
-        or syntax_error "$Command: Missing CHANGE.";
+    my @args;
+    push @args, '--edit'      if $Options{edit};
+    push @args, '--no-commit' if $Options{'no-commit'};
 
-    # Make sure we haven't popped out an option.
-    $id !~ /^-/
-        or syntax_error "$Command: Missing CHANGE.";
+    @ARGV or syntax_error "$Command: Missing CHANGE.";
 
-    my $change = get_change($id);
+    my @change_branches = do {
+        local $Command = 'fetch';
+        $Commands{fetch}->();
+    };
 
-    my $project = config('project');
-    $change->{project} eq $project
-        or error "$Command: Change $id belongs to a different project ($change->{project}), not $project";
-
-    my ($revision) = values %{$change->{revisions}};
-
-    my ($url, $ref) = @{$revision->{fetch}{http}}{qw/url ref/};
-
-    cmd "git fetch $url $ref"
-        or error "$Command: can't git fetch $url $ref";
-
-    cmd join(' ', 'git cherry-pick', @ARGV, 'FETCH_HEAD');
+    cmd join(' ', 'git cherry-pick', @args, @change_branches);
 
     return;
 };
@@ -942,36 +959,38 @@ $Commands{reviewer} = sub {
         'delete=s@',
     );
 
-    my $id = shift @ARGV || current_change_id()
-        or syntax_error "$Command: Missing CHANGE.";
+    grok_unspecified_change;
 
-    # First try to make all deletions
-    if (my $users = $Options{delete}) {
-        foreach my $user (split(/,/, join(',', @$users))) {
-            $user = uri_escape_utf8($user);
-            gerrit_or_die(DELETE => "/changes/$id/reviewers/$user");
+    foreach my $id (@ARGV) {
+        # First try to make all deletions
+        if (my $users = $Options{delete}) {
+            foreach my $user (split(/,/, join(',', @$users))) {
+                $user = uri_escape_utf8($user);
+                gerrit_or_die(DELETE => "/changes/$id/reviewers/$user");
+            }
         }
-    }
 
-    # Second try to make all additions
-    if (my $users = $Options{add}) {
-        my $confirm = $Options{confirm} ? 'true' : 'false';
-        foreach my $user (split(/,/, join(',', @$users))) {
-            gerrit_or_die(POST => "/changes/$id/reviewers", { reviewer => $user, confirm => $confirm});
+        # Second try to make all additions
+        if (my $users = $Options{add}) {
+            my $confirm = $Options{confirm} ? 'true' : 'false';
+            foreach my $user (split(/,/, join(',', @$users))) {
+                gerrit_or_die(POST => "/changes/$id/reviewers", { reviewer => $user, confirm => $confirm});
+            }
         }
+
+        # Finally, list current reviewers
+        my $reviewers = gerrit_or_die(GET => "/changes/$id/reviewers");
+
+        print "[$id]\n";
+        require Text::Table;
+        my %labels = map {$_ => undef} map {keys %{$_->{approvals}}} @$reviewers;
+        my @labels = sort keys %labels;
+        my $table = Text::Table->new('REVIEWER', map {"$_\n&num"} @labels);
+        $table->add($_->{name}, @{$_->{approvals}}{@labels})
+            foreach sort {$a->{name} cmp $b->{name}} @$reviewers;
+        print $table->table(), '-' x 60, "\n";
     }
 
-    # Finally, list current reviewers
-    my $reviewers = gerrit_or_die(GET => "/changes/$id/reviewers");
-
-    require Text::Table;
-    my %labels = map {$_ => undef} map {keys %{$_->{approvals}}} @$reviewers;
-    my @labels = sort keys %labels;
-    my $table = Text::Table->new('REVIEWER', map {"$_\n&num"} @labels);
-    $table->add($_->{name}, @{$_->{approvals}}{@labels})
-        foreach sort {$a->{name} cmp $b->{name}} @$reviewers;
-    print $table->table(), "\n";
-
     return;
 };
 
@@ -1000,21 +1019,15 @@ $Commands{review} = sub {
     error "$Command: You must specify a message or a vote to review."
         unless keys %review;
 
-    if (my $id = shift @ARGV) {
+    my $local_change = grok_unspecified_change;
+
+    foreach my $id (@ARGV) {
         gerrit_or_die(POST => "/changes/$id/revisions/current/review", \%review);
-    } else {
-        my $branch = current_branch;
-
-        my ($upstream, $id) = change_branch_info($branch)
-            or error "$Command: Missing CHANGE.";
-
-        gerrit_or_die(POST => "/changes/$id/revisions/current/review", \%review);
-
-        unless ($Options{keep}) {
-            cmd "git checkout $upstream" and cmd "git branch -D $branch";
-        }
     }
 
+    checkout_upstream_and_delete_branch
+        if $local_change && ! $Options{keep};
+
     return;
 };
 
@@ -1030,37 +1043,32 @@ $Commands{abandon} = sub {
         push @args, { message => $message };
     }
 
-    if (my $id = shift @ARGV) {
+    my $local_change = grok_unspecified_change;
+
+    foreach my $id (@ARGV) {
         gerrit_or_die(POST => "/changes/$id/abandon", @args);
-    } else {
-        my $branch = current_branch;
-
-        my ($upstream, $id) = change_branch_info($branch)
-            or error "$Command: Missing CHANGE.";
-
-        gerrit_or_die(POST => "/changes/$id/abandon", @args);
-
-        unless ($Options{keep}) {
-            cmd "git checkout $upstream" and cmd "git branch -D $branch";
-        }
     }
 
+    checkout_upstream_and_delete_branch
+        if $local_change && ! $Options{keep};
+
     return;
 };
 
 $Commands{restore} = sub {
     get_options('message=s');
 
-    my $id = shift @ARGV || current_change_id()
-        or syntax_error "$Command: Missing CHANGE.";
-
-    my @args = ("/changes/$id/restore");
+    my @args;
 
     if (my $message = get_message) {
         push @args, { message => $message };
     }
 
-    gerrit_or_die(POST => @args);
+    grok_unspecified_change;
+
+    foreach my $id (@ARGV) {
+        gerrit_or_die(POST => "/changes/$id/restore", @args);
+    }
 
     return;
 };
@@ -1068,16 +1076,17 @@ $Commands{restore} = sub {
 $Commands{revert} = sub {
     get_options('message=s');
 
-    my $id = shift @ARGV || current_change_id()
-        or syntax_error "$Command: Missing CHANGE.";
-
-    my @args = ("/changes/$id/revert");
+    my @args;
 
     if (my $message = get_message) {
         push @args, { message => $message };
     }
 
-    gerrit_or_die(POST => @args);
+    grok_unspecified_change;
+
+    foreach my $id (@ARGV) {
+        gerrit_or_die(POST => "/changes/$id/revert", @args);
+    }
 
     return;
 };
@@ -1091,21 +1100,15 @@ $Commands{submit} = sub {
     my @args;
     push @args, { wait_for_merge => 'true' } unless $Options{'no-wait-for-merge'};
 
-    if (my $id = shift @ARGV) {
+    my $local_change = grok_unspecified_change;
+
+    foreach my $id (@ARGV) {
         gerrit_or_die(POST => "/changes/$id/submit", @args);
-    } else {
-        my $branch = current_branch;
-
-        my ($upstream, $id) = change_branch_info($branch)
-            or error "$Command: Missing CHANGE.";
-
-        gerrit_or_die(POST => "/changes/$id/submit", @args);
-
-        unless ($Options{keep}) {
-            cmd "git checkout $upstream" and cmd "git branch -D $branch";
-        }
     }
 
+    checkout_upstream_and_delete_branch
+        if $local_change && ! $Options{keep};
+
     return;
 };
 
@@ -1129,7 +1132,7 @@ $Commands{web} = sub {
         }
     }
 
-    grok_unspecified_change();
+    grok_unspecified_change;
 
     # Grok the URLs of each change
     my @urls;
@@ -1195,7 +1198,7 @@ App::GitGerrit - A container for functions for the git-gerrit program
 
 =head1 VERSION
 
-version 0.020
+version 0.021
 
 =head1 SYNOPSIS
 
